#!/usr/bin/env node

var path = require("path"), fs = require("fs");
var childProcess = require("child_process");
var color = require("ansi-color").set;
var ideServer = require("../test_suite/server/server.js");
var cukeTree = require("./cukeTree.js");

var staticConfig = { idePort : 8222 };
var defaultConfigFile = path.join(process.cwd, "default");

var cli = getCli();
var argv = cli.argv;

if (argv.h) { 
	cli.showHelp();
} else {
	getConfiguration(argv, function(err, config) {
		if (err) {
			console.error(color(err, "red"));
			cli.showHelp()
			process.exit(9);
		} else {
			switch (config.command) {
				case "run":
					generateReport(config, function(err) {
						if (err) {
							console.error("An error occurred generating the report...");
							console.error(err);
							process.exit(1);
						} else {
							launchAction(config);
						}
					});
					break;
				case "ide":
					startIde(config, function(err) {
						if (err) {
							console.error("An error occurred running the ide...");
							console.error(err);
							process.exit(1);
						} else {
							launchAction(config);
						}
					});
					break;
				default:
					break;
			}
		}
	});		
}


function getCli() {
	var usage = "\nGenerates a CukeTree report.\n\nUsage: $0 <command> -i <filename> -o <directory> -f <directory>\n";
	usage+="\nCommands:\n  RUN - Runs cuketree\n  IDE - Starts cuketree ide\n";

	return require("optimist")
		.usage(usage)
		.alias("h", "help").describe("h", "Optional. Shows usage information")
		.alias("c", "config").describe("c", "Optional. Name of the configuration profile for running cuketree")
		.alias("i", "input").describe("i", "Name of cucumber test data (as JSON) used to generate report")
		.alias("o", "output").describe("o", "Output directory to generate report")
		.alias("f", "features").describe("f", "Features directory the report was generated from")
		.alias("l", "launch").describe("l", "Optional. Launch/open the report/ide with the default handler")
		.alias("b", "bin").describe("b", "Optional. Bin location of cucumber-js")
		.alias("r", "run").describe("r", "Optional. Location of test directory to execute");
}

function getConfiguration(argv, callback) {
	var command = argv._[0] ? argv._[0].toLowerCase() : null;
	
	var configFile = argv.config;
	if (!configFile) {
		configFile = defaultConfigFile;
	}
	configFile = path.resolve(process.cwd, configFile);
	if (path.extname(configFile) === "") {
		configFile += ".cukeTree.js";
	}
	
	if (!fs.existsSync(configFile)) {
		callback("CukeTree config file does not exist: "+ configFile);
	}
	
	var config = {};
	if (configFile) {
		var configPath = path.dirname(configFile);
		config = fs.readFileSync(configFile);
		config = JSON.parse(config);
		Object.keys(config).forEach(function(key) {
			if (["input", "output", "features", "run"].indexOf(key) > -1) {
				config[key] = path.resolve(configPath, config[key]);
			}
		});
	}
	config = {
		command : command ? command : config.command,
		input : argv.input ? path.resolve(process.cwd, argv.input) : config.input,
		output : argv.output ? path.resolve(process.cwd, argv.output) : config.output,
		features : argv.features ? path.resolve(process.cwd, argv.features) : config.features,
		bin : argv.bin ? argv.bin : config.bin,
		launch : argv.launch ? argv.launch : config.launch,
		run : argv.run
			? argv.run != true ? argv.run : null
			: config.run
	};
	
	var commandIndex = ["run", "ide"].indexOf(config.command);		
	if (commandIndex == -1) {
		callback("Command not recognised : "+ config.command);
	} else if (!config.input || !config.output || !config.features) {
		var error = "Missing one or more required parameters to generate report";
		error += JSON.stringify(config, null, 3);
		callback(error);
	} else {
		callback(null, config);
	}
}

function launchAction(config) {
	if (config.launch) {
		var execCommand = "start ";
		switch (config.command) {
			case "run":
				execCommand += path.join(config.output, "report.htm");
				break;
			case "ide":
				execCommand += "http://localhost:"+ staticConfig.idePort;
				break;
			default:
				break;
		}
		childProcess.exec(execCommand);
	}
}

function generateReport(config, callback) {
	if (config.run) {
		cukeTree.runTests({
			cucumberBin : config.bin,
			output : config.input, //cukeTree input (report.json)
			features : config.run, 
			finishedCallback : cucumberProcessFinished,
			exitedCallback : function(code) {
				if (code === 0 || code === 1) {
					runReport();
				} else {
					var err = "Tests failed and exited with code : " + code;
					console.log(err);
				}	
			}
		});
	} else {
		runReport();
	}
	
	function runReport() {
		cukeTree.generateReport({
			input : config.input,
			output : config.output,
			features : config.features
		}, callback);
	}
}

function startIde(config, callback) {
	if (!config.bin) {
		console.error("Error. You must specify a bin location for cucumber");
		process.exit(1);
	} else {
		ideServer.start({
			port : staticConfig.idePort,
			featuresPath : config.features,
			glossaryFile : path.join(config.features, "glossary.js"),
			resultsPath : path.dirname(config.input),
			cucumberBin : config.bin
		}, callback);
	}
}

function cucumberProcessFinished(error, stdout, stderr) {
	console.log(color("\n------------------------------------------------------------", "black+white_bg"));
	if (error) { 
		console.log(color("Child Process Error:-", "red+bold"));
		console.log(color("Error code: ", "yellow") +error.code);
		console.log(color("Signal received: ", "yellow") +error.signal);
		console.log(color("Stack:- ", "yellow+bold"));
		console.log(error.stack);
		console.log(color("STDERR:-\n", "red+bold") +stderr);
		console.log(color("STDOUT:-\n", "green+bold") +stdout);
	} else {
		console.log(stdout);
	}				
	console.log(color("------------------------------------------------------------", "black+white_bg"));
}